package jp.ac.kagawa_u.eng.wappen.actions;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;

import jp.ac.kagawa_u.eng.wappen.Activator;
import jp.ac.kagawa_u.eng.wappen.util.Service;
import jp.ac.kagawa_u.eng.wappen.util.Upload;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IPathEditorInput;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

public class UploadAction implements IWorkbenchWindowActionDelegate {
	private IWorkbenchWindow window;

	public void dispose() {
	}

	public void init(IWorkbenchWindow window) {
		this.window = window;
	}

	public void selectionChanged(IAction action, ISelection selection) { 
		IProject project;
		try {
			IPathEditorInput input = (IPathEditorInput)window.getActivePage().getActiveEditor().getEditorInput();
			IFile file = (IFile)input.getAdapter(IFile.class);
			project = file.getProject();
		} catch (NullPointerException e) {
			return;
		}
		if (project==null) {
			return;
		}
			
		URL uploadURL = null;
		try {
			uploadURL = (URL)(project.getSessionProperty(new QualifiedName(null, "__upload")));
		} catch (CoreException e) {}

		action.setEnabled(uploadURL!=null);
	}

	public void run(IAction action) {
		IPathEditorInput input = (IPathEditorInput)window.getActivePage().getActiveEditor().getEditorInput();
		ILog ilog = ResourcesPlugin.getPlugin().getLog();

		String[] targets;
		URL uploadURL;
		IFile file;
		IProject project;
		ArrayList<IFile> sources;
		String encoding = null;
		try {
			file = (IFile)input.getAdapter(IFile.class);
			project = file.getProject();

			sources = (ArrayList<IFile>)project.getSessionProperty(new QualifiedName(null, "__sources"));		
			String targetsStr = (String)project.getSessionProperty(new QualifiedName(null, "targets"));
			encoding = (String)project.getSessionProperty(new QualifiedName(null, "encoding"));
			if (encoding!=null && encoding.trim().length()==0) {
				encoding = null;
			}
			if (targetsStr==null) {
				targets = new String[] {};
			} else {
				targets = targetsStr.split("\\s+");
			}
			uploadURL = (URL)(project.getSessionProperty(new QualifiedName(null, "__upload")));

			if (uploadURL==null) {
				MessageDialog.openInformation(null,
						"", //$NON-NLS-1$
						"Abv[hpURLݒ肳Ă܂B");
				return;
			}
		} catch (CoreException e) {
			ilog.log(new Status(Status.ERROR, Activator.PLUGIN_ID, Status.OK, e.toString(), e)); 
			return;
		}

		uploadFileByExternalBrowser(project, sources, targets, uploadURL, encoding);
	}

	private void uploadFile(IProject project, ArrayList<IFile> sources, String[] targets, URL uploadURL, String encoding) {
		ILog ilog = ResourcesPlugin.getPlugin().getLog();
		try {      
			System.err.printf("uploadURL=%s%n", uploadURL);
			URLConnection urlCon = uploadURL.openConnection();

			String cookies = (String)(project.getSessionProperty(new QualifiedName(null, "cookies")));
			if (cookies!=null) {
				for(String cookie : cookies.split(" "))
					urlCon.addRequestProperty("cookie", cookie);
			}

			urlCon.setDoOutput(true);
//			urlCon.setDoInput(true); // defaultȂ̂ŏȗ
			urlCon.setUseCaches(false);

			String boundary = Upload.defaultBoundary;
			urlCon.setRequestProperty("Content-Type", String.format("multipart/form-data; boundary=%s", boundary));

			byte[] bytes = new byte[1024];
			
			DataOutputStream dos = new DataOutputStream(urlCon.getOutputStream());
			for(IFile src : sources) {
				IPath path = src.getLocation();
				String directory = path.removeLastSegments(1).toString();
				String filename = path.lastSegment();
				String projectpath = src.getProjectRelativePath().toString();
				Upload.uploadSingleFile(directory, filename, projectpath, "text/plain", dos, boundary, bytes, encoding);
			}
			
			for(String target : targets) {
				IPath path = project.getFile(target).getLocation();
				String directory = path.removeLastSegments(1).toString();
				String filename = path.lastSegment();
				Upload.uploadSingleFile(directory, filename, target, "application/octet-stream", dos, boundary, bytes, null);
			}
			dos.writeBytes(String.format("\r\n--%s--\r\n", boundary));
			dos.close();

			BufferedReader br = new BufferedReader(new InputStreamReader(urlCon.getInputStream()));
			String redirectURL = br.readLine();
			if (redirectURL.equals("success")) {
				redirectURL = br.readLine();
				Service.openBrowser(redirectURL);
			} else {
				StringBuilder sb = new StringBuilder();
				sb.append(redirectURL);
				System.err.printf("%s%n", redirectURL);
				while(true) {
					String line = br.readLine();
					if (line==null) break;
					sb.append(line);
					System.err.printf("%s%n", line);
				}
				MessageDialog.openInformation(null,
						"", //$NON-NLS-1$
						"Abv[hɎs܂B\nR\r"+sb.toString());
			}
			br.close();
		} catch (MalformedURLException e) {
			e.printStackTrace(System.err);
			ilog.log(new Status(Status.ERROR, Activator.PLUGIN_ID, Status.OK, e.toString(), e));     
		} catch (CoreException e) {
			e.printStackTrace(System.err);
			ilog.log(new Status(Status.ERROR, Activator.PLUGIN_ID, Status.OK, e.toString(), e));
		} catch (IOException e) {
			e.printStackTrace(System.err);
			ilog.log(new Status(Status.ERROR, Activator.PLUGIN_ID, Status.OK, e.toString(), e));
		}
	}
	
	/**
	 * @param project
	 * @param sources
	 * @param targets
	 * @param uploadURL
	 */
	private void uploadFileByExternalBrowser(IProject project, ArrayList<IFile> sources, String[] targets, URL uploadURL, String encoding) {
		ILog ilog = ResourcesPlugin.getPlugin().getLog();
		try {
			File tmpfile = File.createTempFile("wappenUpload", ".html");
			System.err.printf("tmpfile at %s%n", tmpfile.getAbsolutePath());
			PrintWriter out = new PrintWriter(new OutputStreamWriter(new FileOutputStream(tmpfile), encoding));
			// TODO encoding킹
			out.printf("<html>%n");
			out.printf("<head>");
			out.printf("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\"%n", encoding);
			out.printf("<title>Uploading files</title></head>%n");
			out.printf("<body>%n");
			out.printf("<h1>t@C̃Abv[h</h1>%n");
			out.printf("<form action='%s' method='post'>%n", uploadURL);
			out.printf("<input type='submit' value='M' /><br/>%n");
			for(IFile src : sources) {
				String projectpath = src.getProjectRelativePath().toString();

				out.printf("<b>%s</b><br/>%n", projectpath);	
				StringBuilder sb = new StringBuilder();
				String line;
				int lines=0;
				BufferedReader in = new BufferedReader(new InputStreamReader(src.getContents(), src.getCharset()));
				while((line=in.readLine())!=null) {
					line = line.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;");
					lines++;
					sb.append(line).append("\n");
				}
				out.printf("<textarea name='%s' cols='256' rows='%d'>", projectpath, lines+1);
				out.print(sb.toString());
				out.printf("</textarea><br/>%n");
			}
			
			for (String target : targets) {
				try {
					IFile file = project.getFile(target);
					file.refreshLocal(IResource.DEPTH_ZERO, null);
					InputStream is = file.getContents(true);
					ByteArrayOutputStream os = new ByteArrayOutputStream();
					util.Base64IO.encode(is, os, true);
					out.printf("<i>(%s)</i><br/>%n", target);
					out.printf("<textarea name='%s.B64' style='border-style:none;width:0;height:1'>\r\n", target);
					out.print(os.toString("ISO-8859-1"));
					out.printf("</textarea><br/>%n");
				} catch (CoreException e) {
					e.printStackTrace(System.err);
					ilog.log(new Status(Status.ERROR, Activator.PLUGIN_ID, Status.OK, e.toString(), e));
				} catch (IOException e) {
					e.printStackTrace(System.err);
					ilog.log(new Status(Status.ERROR, Activator.PLUGIN_ID, Status.OK, e.toString(), e));
				}	
			}
			out.printf("<input type='submit' value='M' />%n");
			out.printf("</form>%n");
			out.printf("<hr/>%n");
			out.printf("</body>%n");
			out.printf("</html>%n");
			out.close();
			Service.viewURL("Uploading files...", tmpfile.toURI().toURL());
		} catch (IOException e) {
			e.printStackTrace(System.err);
			ilog.log(new Status(Status.ERROR, Activator.PLUGIN_ID, Status.OK, e.toString(), e));
		} catch (CoreException e) {
			e.printStackTrace(System.err);
			ilog.log(new Status(Status.ERROR, Activator.PLUGIN_ID, Status.OK, e.toString(), e));
		}
		
	}

}
